import React, { useEffect, useState } from 'react';
import { Grid, Typography, Card, CardContent, Container, Link as MuiLink, makeStyles } from '@material-ui/core';
import { DeploymentStepper } from "./DeploymentStepper"
import { KeyValueTable } from "../common/KeyValueTable/KeyValueTable";
import { SimpleAccordion } from "../common/Accordion/SimpleAccordion";
import { DeploymentBadge } from './DeploymentBadge';
import { DeploymentJobOutput } from './DeploymentJobOutput';
import { Alert } from '@material-ui/lab';
import { Link } from 'react-router-dom';
import { formatDate } from '../common/Helper';
import GitRefLink from '../IOC/GitRefLink';
import AccessControl from '../auth/AccessControl';
import { DeploymentStatus } from '../../api/DataTypes';
import { theme } from "../../style/Theme";
import AlertMessages from '../IOC/AlertMessages';

function createAlert(deployment, deploymentJob) {
  const deploymentStatus = new DeploymentStatus(deployment, deploymentJob);
  const message = deploymentStatus.message();
  const severity = deploymentStatus.severity();
  return <Alert severity={severity} variant="standard">{message}</Alert>;
}

const useStyles = makeStyles((theme) => ({
  csentryIdInvalid: {
    color: theme.csentryIdInvalid.color
  }
}));

export function DeploymentDetails({ deployment, deploymentJob }) {
  const [alert, setAlert] = useState(createAlert(deployment, deploymentJob));

  const classes = useStyles(theme);

  let deplAlert = deployment.alerts;

  useEffect(() => {
    setAlert(createAlert(deployment, deploymentJob));

    //show SUCCESS message only to successful deployments (where no warnings came back from BE)
    if (deployment && deplAlert?.length === 0 && !deployment.undeployment && deploymentJob?.status.toLowerCase() === 'successful') {
      deplAlert.push({type: 'SUCCESS', message: 'Deployment was successful!'});
    }

  }, [deployment, deploymentJob])

  function calculateHostText() {
    //host is resolvable => show link for users
    if (deployment.host.csEntryId && deployment.host.csEntryIdValid) {
      return (
      <Typography>
          <Link to={`/hosts/${deployment.host.csEntryId}`}>{deployment.host.fqdn}</Link>
      </Typography>)
    }

    //gray out hostname if it is from cache
    return { shownValue: 
      <Typography>
        {deployment.host.fqdn}
      </Typography>, shownClass: classes.csentryIdInvalid };
  }

  const deploymentDetails = {
    "Revision": <GitRefLink deployment={deployment} />,
    comment: deployment.comment?.length > 0 ? deployment.comment : "---",
    host:
      calculateHostText(),
    "user": deployment.createdBy,
    repository: <Typography><MuiLink href={deployment.sourceUrl} target="_blank" rel="noreferrer">{deployment.sourceUrl}</MuiLink></Typography>,
    "AWX job link": <Typography><MuiLink href={deployment.awxJobUrl} target="_blank" rel="noreferrer">{deployment.awxJobUrl}</MuiLink></Typography>,
    "start time": deploymentJob?.started ? formatDate(deploymentJob.started) : "-",
  }

/*{alert && <Grid item xs={12}>
          <AlertMessages alerts={liveIOC.alerts} />
        </Grid>}*/

  return (
    <Grid container spacing={1}>
     {deplAlert && <Grid item xs={12}>
          <AlertMessages alerts={deplAlert} />
        </Grid>}
      <Grid item xs={12} md={12}>
        <SimpleAccordion defaultExpanded summary={<DeploymentBadge deployment={deployment} />}>
          <KeyValueTable obj={deploymentDetails} variant="overline" />
        </SimpleAccordion>
      </Grid>
      <Grid item xs={12} md={12}>
        <Card >
          <CardContent>
            <div>
              {deploymentJob &&
                <DeploymentStepper
                  activeStep={["created", "pending", "running", "successful"].indexOf(deploymentJob.status.toLowerCase())}
                  deploymentStatus={deploymentJob.status}
                  deploymentStart={deployment.startDate}
                  undeployment={deployment.undeployment}
                />}
            </div>
          </CardContent>
        </Card>
      </Grid>
      {alert && <Grid item xs={12}>
        {alert}
      </Grid>}
      <AccessControl allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
        renderNoAccess={() => <></>}>
        <Grid item xs={12} md={12} style={{ paddingBottom: 0 }}>
          {deploymentJob ?
            <SimpleAccordion summary="Ansible Job Output" defaultExpanded>
              <Container>
                <DeploymentJobOutput deploymentJob={deploymentJob} />
              </Container>
            </SimpleAccordion>
            :
            <></>
          }
        </Grid>
      </AccessControl>
    </Grid>
  );
}

